/*
* Copyright 2006 Sony Computer Entertainment Inc.
*
* Licensed under the MIT Open Source License, for details please see license.txt or the website
* http://www.opensource.org/licenses/mit-license.php
*
*/
#ifndef _CFX_PS3_H_
#define _CFX_PS3_H_

#include <PSGL/psgl.h>

#define CG_FILE_TYPE CG_BINARY

inline void cfxPrint(const char* str, ...)
{
	printf(str);
}

#endif
