/*
* Copyright 2006 Sony Computer Entertainment Inc.
*
* Licensed under the MIT Open Source License, for details please see license.txt or the website
* http://www.opensource.org/licenses/mit-license.php
*
*/ 
#ifndef _CRT_PLATFORM_H_
#define _CRT_PLATFORM_H_

// COLLADA_RT platform includes
// The PLATFORM_INCLUDE define specifies the header file which includes all platform specific information
// For windows projects this is \"Crt/CrtWin.h\" which makes this statement include "Crt/CrtWin.h"
#include CRT_PLATFORM_INCLUDE

#endif // _CRT_PLATFORM_H_

