/*
* Copyright 2006 Sony Computer Entertainment Inc.
*
* Licensed under the MIT Open Source License, for details please see license.txt or the website
* http://www.opensource.org/licenses/mit-license.php
*
*/ 
#include "Crt/CrtLight.h"
#include "Crt/CrtRender.h"
#include "Crt/CrtUtils.h"

void	CrtLight::Update( CrtInt nbrLights)
{
(void)nbrLights;  // avoids warnings
}								
